@@ -535,8 +535,8 @@ class TestAbstractGenerator(unittest.TestCase):
 
     def test_is_derivation(self):
         self.assertFalse(self.generator.extension(self.schema.types['type1']))
-        self.assertFalse(self.generator.extension(self.schema.types['type1'], 'tns:type1'))
-        self.assertFalse(self.generator.restriction(self.schema.types['type1'], 'tns:type1'))
+        self.assertTrue(self.generator.extension(self.schema.types['type1'], 'tns:type1'))
+        self.assertTrue(self.generator.restriction(self.schema.types['type1'], 'tns:type1'))
         self.assertTrue(self.generator.derivation(self.schema.types['type1'], 'type1'))
         self.assertFalse(self.generator.restriction(self.schema.types['type6'], 'xs:decimal'))
         self.assertFalse(self.generator.restriction(self.schema.types['type6'], None))
@@ -634,9 +634,5 @@ class TestPythonGenerator11(TestPythonGenerator):
 
 
 if __name__ == '__main__':
-    import platform
-    header_template = "Test xmlschema code generators with Python {} on {}"
-    header = header_template.format(platform.python_version(), platform.platform())
-    print('{0}\n{1}\n{0}'.format("*" * len(header), header))
-
-    unittest.main()
+    from xmlschema.testing import run_xmlschema_tests
+    run_xmlschema_tests('code generators')
