@@ -18,10 +18,10 @@ from xmlschema import XMLSchema10, XMLSchema11, fetch_namespaces, etree_tostring
     XsdElement, XsdAttribute, XsdType
 
 from xmlschema.validators import XsdAttributeGroup
-from xmlschema.helpers import is_etree_element
+from xmlschema.utils.etree import is_etree_element
 from xmlschema.names import XSI_TYPE
 from xmlschema.dataobjects import DataBindingMeta, DataBindingConverter
-from xmlschema.testing import etree_elements_assert_equal
+from xmlschema.testing import etree_elements_assert_equal, run_xmlschema_tests
 
 
 class TestDataElementInterface(unittest.TestCase):
@@ -414,15 +414,15 @@ class TestDataObjects(unittest.TestCase):
         col_data = self.col_schema.decode(self.col_xml_filename)
         self.assertTrue(col_data.is_valid())
 
-        col_data[0][0].value = '1'
+        col_data[0][0].value = '1.0'
         with self.assertRaises(XMLSchemaValidationError) as ec:
             col_data.validate()
-        self.assertIn("'1' is not an instance of <class 'int'>", str(ec.exception))
+        self.assertIn("invalid literal for int() with base 10", str(ec.exception))
         self.assertFalse(col_data.is_valid())
 
         errors = list(col_data.iter_errors())
         self.assertEqual(len(errors), 1)
-        self.assertIn("'1' is not an instance of <class 'int'>", str(errors[0]))
+        self.assertIn("invalid literal for int() with base 10", str(errors[0]))
 
         col_data.find('object/position').value = 1
         self.assertTrue(col_data.is_valid())
@@ -651,9 +651,4 @@ class TestDataBindings11(TestDataBindings):
 
 
 if __name__ == '__main__':
-    import platform
-    header_template = "Test xmlschema data objects with Python {} on {}"
-    header = header_template.format(platform.python_version(), platform.platform())
-    print('{0}\n{1}\n{0}'.format("*" * len(header), header))
-
-    unittest.main()
+    run_xmlschema_tests('data objects')
