@@ -13,18 +13,20 @@ import sys
 import os
 import pathlib
 import platform
+import warnings
 
 from urllib.parse import urlsplit
+from urllib.request import pathname2url
 from pathlib import Path, PurePath, PureWindowsPath, PurePosixPath
 from unittest.mock import patch, MagicMock
 
-import xmlschema.locations
-from xmlschema.locations import LocationPath, LocationPosixPath, LocationWindowsPath, \
-    is_url, is_local_url, is_remote_url, url_path_is_file, is_unc_path, is_drive_path, \
-    normalize_url, normalize_locations, match_location, is_encoded_url, is_safe_url, \
-    encode_url, decode_url, get_uri_path, get_uri, DRIVE_LETTERS
-
-TEST_CASES_DIR = str(pathlib.Path(__file__).absolute().parent.joinpath('test_cases'))
+from xmlschema.testing import XMLSchemaTestCase, run_xmlschema_tests
+from xmlschema.utils.paths import DRIVE_LETTERS, get_uri, get_uri_path, is_unc_path, \
+    is_drive_path, LocationPath, LocationPosixPath, LocationWindowsPath
+from xmlschema.utils.urls import is_url, is_local_url, is_remote_url, is_encoded_url, \
+    is_safe_url, encode_url, decode_url, location_is_file, normalize_url, normalize_locations, \
+    match_location
+import xmlschema.utils.urls
 
 DRIVE_REGEX = '(/[a-zA-Z]:|)' if platform.system() == 'Windows' else ''
 
@@ -37,7 +39,7 @@ URL_CASES = (
     'file:///tmp/xmlschema/schemas/VC/XMLSchema-versioning.xsd',
     'file:///tmp/xmlschema/schemas/XSD_1.1/xsd11-extra.xsd',
     'issue #000.xml', 'dev/XMLSCHEMA/test.xsd',
-    'file:///tmp/xmlschema/schemas/XSI/XMLSchema-instance_minimal.xsd',
+    'file:///tmp/xmlschema/schemas/XSI/XMLSchema-instance.xsd',
     'vehicles.xsd', 'file://filer01/MY_HOME/',
     '//anaconda/envs/testenv/lib/python3.6/site-packages/xmlschema/validators/schemas/',
     'z:\\Dir-1.0\\Dir-2_0\\', 'https://host/path?name=2&id=', 'data.xml',
@@ -50,11 +52,11 @@ URL_CASES = (
     '/tmp/tests/test_cases/examples/collection', 'file:other.xsd',
     'issue%20%23000.xml', '\\\\filer01\\MY_HOME\\dev\\XMLSCHEMA\\test.xsd',
     'http://site/base', 'dir2/schema.xsd', '//root/dir1/schema.xsd',
-    'file:///tmp/xmlschema/schemas/XML/xml_minimal.xsd',
+    'file:///tmp/xmlschema/schemas/XML/xml.xsd',
     'https://site/base', 'file:///home#attribute',
     '/dir1/dir2/issue%20%23002', '////root/dir1/schema.xsd',
     '/tmp/xmlschema/schemas/XSD_1.1/XMLSchema.xsd',
-    '/tmp/xmlschema/schemas/XML/xml_minimal.xsd',
+    '/tmp/xmlschema/schemas/XML/xml.xsd',
     '/tmp/xmlschema/schemas/XSD_1.0/XMLSchema.xsd',
     'file:///home/', '////root/dir1', '//root/dir1/', 'file:///home', 'other.xsd',
     'file:///tmp/tests/test_cases/examples/collection/collection.xml',
@@ -64,7 +66,7 @@ URL_CASES = (
     'https://xmlschema.test/schema 2/test.xsd?name=2 id=3',
     'xsd1.0/schema.xsd', '/home', 'schema.xsd',
     'dev\\XMLSCHEMA\\test.xsd', '../dir1/./dir2', 'beta',
-    '/tmp/xmlschema/schemas/XSI/XMLSchema-instance_minimal.xsd',
+    '/tmp/xmlschema/schemas/XSI/XMLSchema-instance.xsd',
     'file:///dir1/dir2/', 'file:///dir1/dir2/issue%20001', '/root/dir1/schema.xsd',
     'file:///tmp/xmlschema/schemas/XSD_1.1', '/path/schema 2/test.xsd?name=2 id=3',
     'file:////filer01/MY_HOME/', 'file:///home?name=2&id=', 'http://example.com/beta',
@@ -72,10 +74,6 @@ URL_CASES = (
 )
 
 
-def casepath(relative_path):
-    return str(pathlib.Path(TEST_CASES_DIR).joinpath(relative_path))
-
-
 def is_windows_path(path):
     """Checks if the path argument is a Windows platform path."""
     return '\\' in path or ':' in path or '|' in path
@@ -94,13 +92,25 @@ def filter_windows_path(path):
         return path
 
 
-class TestLocations(unittest.TestCase):
+# noinspection PyArgumentList
+def uri_from_path(path, suppress_warning=True):
+    if sys.version_info >= (3, 14) and not suppress_warning:
+        return pathname2url(str(path), add_scheme=True)  # The result is different!!
+
+    with warnings.catch_warnings():
+        warnings.simplefilter("ignore")
+        return path.as_uri()
+
+
+class TestLocations(XMLSchemaTestCase):
+
+    cases_dir = pathlib.Path(__file__).absolute().parent.joinpath('test_cases')
 
     @classmethod
     def setUpClass(cls):
-        cls.col_dir = casepath('examples/collection')
-        cls.col_xsd_file = casepath('examples/collection/collection.xsd')
-        cls.col_xml_file = casepath('examples/collection/collection.xml')
+        cls.col_dir = cls.casepath('examples/collection')
+        cls.col_xsd_file = cls.casepath('examples/collection/collection.xsd')
+        cls.col_xml_file = cls.casepath('examples/collection/collection.xml')
 
     def check_url(self, url, expected):
         url_parts = urlsplit(url)
@@ -159,9 +169,8 @@ class TestLocations(unittest.TestCase):
         else:
             default_class = LocationPosixPath
 
-        with self.assertRaises(ValueError) as ec:
-            LocationPath.from_uri('')
-        self.assertEqual(str(ec.exception), 'Empty URI provided!')
+        path = LocationPath.from_uri('')
+        self.assertEqual(str(path), '.')
 
         path = LocationPath.from_uri('https://example.com/names/?name=foo')
         self.assertIsInstance(path, LocationPosixPath)
@@ -169,14 +178,14 @@ class TestLocations(unittest.TestCase):
 
         path = LocationPosixPath.from_uri('file:///home/foo/names/?name=foo')
         self.assertIsInstance(path, default_class)
-        self.assertEqual(str(path).replace('\\', '/'), '/home/foo/names')
+        self.assertEqual(str(path).replace('\\', '/'), '/home/foo/names/?name=foo')
 
         path = LocationPosixPath.from_uri('file:///home/foo/names#foo')
         self.assertIsInstance(path, default_class)
-        self.assertEqual(str(path).replace('\\', '/'), '/home/foo/names')
+        self.assertEqual(str(path).replace('\\', '/'), '/home/foo/names#foo')
 
         path = LocationPath.from_uri('file:///home\\foo\\names#foo')
-        self.assertTrue(path.as_posix().endswith('/home/foo/names'))
+        self.assertTrue(path.as_posix().endswith('/home/foo/names#foo'))
         self.assertIsInstance(path, LocationWindowsPath)
 
         path = LocationPosixPath.from_uri('file:///c:/home/foo/names/')
@@ -210,7 +219,7 @@ class TestLocations(unittest.TestCase):
         self.assertEqual(get_uri_path('https', 'host', 'path', 'id=7', 'types'),
                          '//host/path')
         self.assertEqual(get_uri_path('k', '', 'path/file', 'id=7', 'types'),
-                         'path/file')
+                         'path/file?id=7#types')
         self.assertEqual(get_uri_path('file', '', 'path/file', 'id=7', 'types'),
                          'path/file')
 
@@ -292,49 +301,52 @@ class TestLocations(unittest.TestCase):
                          f'file:///{base_url}vehicles.xsd')
 
     def test_normalize_url_unc_paths__issue_246(self):
-        url = PureWindowsPath(r'\\host\share\file.xsd').as_uri()
+        url = uri_from_path(PureWindowsPath(r'\\host\share\file.xsd'))
         self.assertNotEqual(normalize_url(r'\\host\share\file.xsd'),
                             url)  # file://host/share/file.xsd
         self.assertEqual(normalize_url(r'\\host\share\file.xsd'),
                          url.replace('file://', 'file:////'))
 
     def test_normalize_url_unc_paths__issue_268(self,):
-        unc_path = r'\\filer01\MY_HOME\dev\XMLSCHEMA\test.xsd'
-        url = PureWindowsPath(unc_path).as_uri()
-        self.assertEqual(str(PureWindowsPath(unc_path)), unc_path)
-        self.assertEqual(url, 'file://filer01/MY_HOME/dev/XMLSCHEMA/test.xsd')
+        with warnings.catch_warnings():
+            warnings.simplefilter("ignore")
 
-        # Same UNC path as URI with the host inserted in path.
-        url_host_in_path = url.replace('file://', 'file:////')
-        self.assertEqual(url_host_in_path, 'file:////filer01/MY_HOME/dev/XMLSCHEMA/test.xsd')
+            unc_path = r'\\filer01\MY_HOME\dev\XMLSCHEMA\test.xsd'
+            url = uri_from_path(PureWindowsPath(unc_path))
+            self.assertEqual(str(PureWindowsPath(unc_path)), unc_path)
+            self.assertEqual(url, 'file://filer01/MY_HOME/dev/XMLSCHEMA/test.xsd')
 
-        self.assertEqual(normalize_url(unc_path), url_host_in_path)
+            # Same UNC path as URI with the host inserted in path.
+            url_host_in_path = url.replace('file://', 'file:////')
+            self.assertEqual(url_host_in_path, 'file:////filer01/MY_HOME/dev/XMLSCHEMA/test.xsd')
 
-        with patch.object(os, 'name', 'nt'):
-            self.assertEqual(os.name, 'nt')
-            path = PurePath(unc_path)
-            self.assertIs(path.__class__, PureWindowsPath)
-            self.assertEqual(path.as_uri(), url)
-
-            self.assertEqual(xmlschema.locations.os.name, 'nt')
-            path = LocationPath(unc_path)
-            self.assertIs(path.__class__, LocationWindowsPath)
-            self.assertEqual(path.as_uri(), url_host_in_path)
             self.assertEqual(normalize_url(unc_path), url_host_in_path)
 
-        with patch.object(os, 'name', 'posix'):
-            self.assertEqual(os.name, 'posix')
-            path = PurePath(unc_path)
-            self.assertIs(path.__class__, PurePosixPath)
-            self.assertEqual(str(path), unc_path)
-            self.assertRaises(ValueError, path.as_uri)  # Not recognized as UNC path
+            with patch.object(os, 'name', 'nt'):
+                self.assertEqual(os.name, 'nt')
+                path = PurePath(unc_path)
+                self.assertIs(path.__class__, PureWindowsPath)
+                self.assertEqual(uri_from_path(path), url)
 
-            self.assertEqual(xmlschema.locations.os.name, 'posix')
-            path = LocationPath(unc_path)
-            self.assertIs(path.__class__, LocationPosixPath)
-            self.assertEqual(str(path), unc_path)
-            self.assertNotEqual(path.as_uri(), url)
-            self.assertEqual(normalize_url(unc_path), url_host_in_path)
+                self.assertEqual(xmlschema.utils.urls.os.name, 'nt')
+                path = LocationPath(unc_path)
+                self.assertIs(path.__class__, LocationWindowsPath)
+                self.assertEqual(path.as_uri(), url_host_in_path)
+                self.assertEqual(normalize_url(unc_path), url_host_in_path)
+
+            with patch.object(os, 'name', 'posix'):
+                self.assertEqual(os.name, 'posix')
+                path = PurePath(unc_path)
+                self.assertIs(path.__class__, PurePosixPath)
+                self.assertEqual(str(path), unc_path)
+                self.assertRaises(ValueError, path.as_uri)  # Not recognized as UNC path
+
+                self.assertEqual(xmlschema.utils.urls.os.name, 'posix')
+                path = LocationPath(unc_path)
+                self.assertIs(path.__class__, LocationPosixPath)
+                self.assertEqual(str(path), unc_path)
+                self.assertNotEqual(path.as_uri(), url)
+                self.assertEqual(normalize_url(unc_path), url_host_in_path)
 
     @unittest.skipIf(platform.system() != 'Windows', "Run only on Windows systems")
     def test_normalize_url_with_base_unc_path_on_windows(self,):
@@ -370,7 +382,7 @@ class TestLocations(unittest.TestCase):
     @unittest.skipIf(platform.system() == 'Windows', "Skip on Windows systems")
     def test_normalize_url_with_base_unc_path_on_others(self,):
         base_unc_path = '\\\\filer01\\MY_HOME\\'
-        base_url = PureWindowsPath(base_unc_path).as_uri()
+        base_url = uri_from_path(PureWindowsPath(base_unc_path))
         self.assertEqual(str(PureWindowsPath(base_unc_path)), base_unc_path)
         self.assertEqual(base_url, 'file://filer01/MY_HOME/')
 
@@ -418,7 +430,7 @@ class TestLocations(unittest.TestCase):
         url = '//anaconda/envs/testenv/lib/python3.6/site-packages/xmlschema/validators/schemas/'
         if os.name == 'posix':
             normalize_url(url)
-            self.assertEqual(normalize_url(url), pathlib.PurePath(url).as_uri())
+            self.assertEqual(normalize_url(url), uri_from_path(pathlib.PurePath(url)))
         else:
             # On Windows // is interpreted as a network share UNC path
             self.assertEqual(os.name, 'nt')
@@ -460,6 +472,9 @@ class TestLocations(unittest.TestCase):
         url = normalize_url('data.xml', '/dir1/dir2/issue%20%23002')
         self.assertRegex(url, f'{DRIVE_REGEX}/dir1/dir2/issue%20%23002/data.xml')
 
+        url = normalize_url('test.#.again.xml', 'file:///dir1/dir2/')
+        self.assertRegex(url, f'{DRIVE_REGEX}/dir1/dir2/test.%23.again.xml')
+
     def test_normalize_url_with_query_part(self):
         url = "https://xmlschema.test/schema 2/test.xsd?name=2 id=3"
         self.assertEqual(
@@ -481,11 +496,11 @@ class TestLocations(unittest.TestCase):
 
         self.assertRegex(
             normalize_url('other.xsd?id=2', 'file:///home?name=2&id='),
-            f'file://{DRIVE_REGEX}/home/other.xsd'
+            f'file://{DRIVE_REGEX}/home%3Fname%3D2%26id%3D/other.xsd%3Fid%3D2'
         )
         self.assertRegex(
             normalize_url('other.xsd#element', 'file:///home#attribute'),
-            f'file://{DRIVE_REGEX}/home/other.xsd'
+            f'file://{DRIVE_REGEX}/home%23attribute/other.xsd%23element'
         )
 
         self.check_url(normalize_url('other.xsd?id=2', 'https://host/path?name=2&id='),
@@ -504,9 +519,7 @@ class TestLocations(unittest.TestCase):
         self.assertEqual(urlsplit(url).geturl(), url)
         self.assertEqual(normalize_url(url), url)
 
-    @unittest.skip
     def test_normalize_url_with_base_url_with_local_part(self):
-
         base_url = "file:///D:/a/xmlschema/xmlschema/filer01/MY_HOME"
         url = normalize_url(r'dev/XMLSCHEMA/test.xsd', base_url)
         self.assertEqual(
@@ -570,13 +583,13 @@ class TestLocations(unittest.TestCase):
         self.assertTrue(is_remote_url(b'  http://example.com/schema.xsd'))
 
     def test_url_path_is_file_function(self):
-        self.assertTrue(url_path_is_file(self.col_xml_file))
-        self.assertTrue(url_path_is_file(normalize_url(self.col_xml_file)))
-        self.assertFalse(url_path_is_file(self.col_dir))
-        self.assertFalse(url_path_is_file('http://example.com/'))
+        self.assertTrue(location_is_file(self.col_xml_file))
+        self.assertTrue(location_is_file(normalize_url(self.col_xml_file)))
+        self.assertFalse(location_is_file(self.col_dir))
+        self.assertFalse(location_is_file('http://example.com/'))
 
         with patch('platform.system', MagicMock(return_value="Windows")):
-            self.assertFalse(url_path_is_file('file:///c:/Windows/unknown'))
+            self.assertFalse(location_is_file('file:///c:/Windows/unknown'))
 
     def test_is_unc_path_function(self):
         self.assertFalse(is_unc_path(''))
@@ -730,8 +743,4 @@ class TestLocations(unittest.TestCase):
 
 
 if __name__ == '__main__':
-    header_template = "Test xmlschema locations.py module with Python {} on platform {}"
-    header = header_template.format(platform.python_version(), platform.platform())
-    print('{0}\n{1}\n{0}'.format("*" * len(header), header))
-
-    unittest.main()
+    run_xmlschema_tests("locations.py module")
