@@ -19,7 +19,7 @@ from elementpath import XPath1Parser, XPath2Parser, Selector, LazyElementNode
 
 from xmlschema import XMLSchema10, XMLSchema11
 from xmlschema.names import XSD_NAMESPACE
-from xmlschema.xpath import XMLSchemaProxy, XPathElement
+from xmlschema.xpath import XMLSchemaProxy, XPathElement, split_path, ElementSelector
 from xmlschema.validators import XsdAtomic, XsdAtomicRestriction
 
 CASES_DIR = os.path.join(os.path.dirname(__file__), 'test_cases/')
@@ -46,7 +46,7 @@ class XMLSchemaProxyTest(unittest.TestCase):
             XMLSchemaProxy(self.xs1, base_element=self.xs2.elements['collection'])
 
         with self.assertRaises(TypeError):
-            XMLSchemaProxy(self.xs1, base_element=ElementTree.Element('vehicles'))
+            XMLSchemaProxy(self.xs1, base_element=ElementTree.Element('vehicles'))  # noqa
 
     def test_bind_parser_method(self):
         schema_proxy1 = XMLSchemaProxy(self.xs1)
@@ -172,6 +172,14 @@ class XPathElementTest(unittest.TestCase):
         elem = XPathElement('foo', self.col_schema.types['objType'])
         self.assertEqual(elem.target_namespace, 'http://example.com/ns/collection')
 
+    def test_xsd_version(self):
+        elem = XPathElement('foo', self.col_schema.types['objType'])
+        self.assertEqual(elem.xsd_version, self.col_schema.xsd_version)
+
+    def test_maps(self):
+        elem = XPathElement('foo', self.col_schema.types['objType'])
+        self.assertIs(elem.maps, self.col_schema.maps)
+
     def test_elem_name(self):
         elem = XPathElement('foo', self.col_schema.types['objType'])
         try:
@@ -314,7 +322,7 @@ class XMLSchemaXPathTest(unittest.TestCase):
         self.assertListEqual(children, self.xs1.elements['vehicles'].type.content[1:])
 
 
-class ElementTreeXPathTest(unittest.TestCase):
+class XPathSelectorsTest(unittest.TestCase):
 
     def test_rel_xpath_boolean(self):
         root = ElementTree.XML('<A><B><C/></B></A>')
@@ -322,11 +330,111 @@ class ElementTreeXPathTest(unittest.TestCase):
         self.assertTrue(Selector('boolean(C)').iter_select(el))
         self.assertFalse(next(Selector('boolean(D)').iter_select(el)))
 
+    def test_split_path(self):
+        path = '/md:EntitiesDescriptor/md:EntityDescriptor[@entityID="https://xmlschema.test"]'
+        result = '/md:EntitiesDescriptor/md:EntityDescriptor[@entityID="https://xmlschema.test"]'
+        self.assertEqual(''.join(split_path(path)), result)
+
+        path = 'md:EntitiesDescriptor/md:EntityDescriptor[@entityID="https://xmlschema.test"]'
+        result = 'md:EntitiesDescriptor/md:EntityDescriptor[@entityID="https://xmlschema.test"]'
+        self.assertEqual(''.join(split_path(path)), result)
+
+        namespaces = {'': 'foo'}
+        self.assertEqual(''.join(split_path(path, namespaces)), result)
+
+        path = '/A/B/C'
+        self.assertEqual(''.join(split_path(path)), '/A/B/C')
+        self.assertEqual(''.join(split_path(path, namespaces)), '/{foo}A/{foo}B/{foo}C')
+
+        path = 'A/B/C'
+        self.assertEqual(''.join(split_path(path)), 'A/B/C')
+        self.assertEqual(''.join(split_path(path, namespaces)), '{foo}A/{foo}B/{foo}C')
+
+        path = 'A/{}B/C[@D="1"]'
+        self.assertEqual(''.join(split_path(path)), 'A/{}B/C[@D="1"]')
+        self.assertEqual(''.join(split_path(path, namespaces)),
+                         '{foo}A/{}B/{foo}C[@D="1"]')
+
+        path = 'A/{bar}B/C[@D="1"]'
+        self.assertEqual(''.join(split_path(path)), 'A/{bar}B/C[@D="1"]')
+        self.assertEqual(''.join(split_path(path, namespaces)),
+                         '{foo}A/{bar}B/{foo}C[@D="1"]')
+
+        path = 'A/p:B/C[@D="1"]/E'
+        self.assertEqual(''.join(split_path(path)), 'A/p:B/C[@D="1"]/E')
+        self.assertEqual(''.join(split_path(path, namespaces)),
+                         '{foo}A/p:B/{foo}C[@D="1"]/{foo}E')
+
+    def test_split_path_parts(self):
+        path = '/md:EntitiesDescriptor/md:EntityDescriptor[@entityID="https://xmlschema.test"]'
+        result = ['/', 'md:EntitiesDescriptor', '/', 'md:EntityDescriptor', '[', '@',
+                  'entityID', '=', '"https://xmlschema.test"', ']']
+        self.assertListEqual(list(split_path(path)), result)
+
+        path = '/A/B/C'
+        result = ['/', '{foo}A', '/', '{foo}B', '/', '{foo}C']
+        namespaces = {'': 'foo'}
+        self.assertEqual(list(split_path(path, namespaces)), result)
+        self.assertEqual(list(split_path(path)), ['/', 'A', '/', 'B', '/', 'C'])
+
+        path = 'A/B/C'
+        self.assertEqual(list(split_path(path)), ['A', '/', 'B', '/', 'C'])
+        self.assertEqual(list(split_path(path, namespaces)), result[1:])
+
+        path = 'A/{}B/C[@D="1"]'
+        result = ['A', '/', '{}B', '/', 'C', '[', '@', 'D', '=', '"1"', ']']
+        self.assertEqual(list(split_path(path)), result)
+        self.assertEqual(list(split_path(path, namespaces)),
+                         ['{foo}A', '/', '{}B', '/', '{foo}C', '[', '@', 'D', '=', '"1"', ']'])
+
+        path = 'A/{bar}B/C[@D="1"]'
+        self.assertEqual(
+            list(split_path(path)),
+            ['A', '/', '{bar}B', '/', 'C', '[', '@', 'D', '=', '"1"', ']']
+        )
+        self.assertEqual(
+            list(split_path(path, namespaces)),
+            ['{foo}A', '/', '{bar}B', '/', '{foo}C', '[', '@', 'D', '=', '"1"', ']']
+        )
+
+        path = 'A/p:B/C[@D="1"]/E'
+        self.assertEqual(
+            list(split_path(path)),
+            ['A', '/', 'p:B', '/', 'C', '[', '@', 'D', '=', '"1"', ']', '/', 'E']
+        )
+        self.assertEqual(
+            list(split_path(path, namespaces)),
+            ['{foo}A', '/', 'p:B', '/', '{foo}C', '[', '@', 'D', '=', '"1"', ']', '/', '{foo}E']
+        )
+
+        self.assertEqual(
+            list(split_path(path, namespaces, extended_names=True)),
+            ['{foo}A', '/', 'p:B', '/', '{foo}C', '[', '@', 'D', '=', '"1"', ']', '/', '{foo}E']
+        )
+
+        namespaces = {'': 'foo', 'p': 'bar'}
+        self.assertEqual(
+            list(split_path(path, namespaces, extended_names=True)),
+            ['{foo}A', '/', '{bar}B', '/', '{foo}C', '[', '@', 'D', '=', '"1"', ']', '/', '{foo}E']
+        )
+
+        path = 'A//p:B/C[2]/E'
+        self.assertEqual(
+            list(split_path(path)),
+            ['A', '//', 'p:B', '/', 'C', '[', '2', ']', '/', 'E']
+        )
+        self.assertEqual(
+            list(split_path(path, namespaces)),
+            ['{foo}A', '//', 'p:B', '/', '{foo}C', '[', '2', ']', '/', '{foo}E']
+        )
+        self.assertEqual(list(split_path('.')), ['.'])
+        self.assertEqual(list(split_path('*/b')), ['*', '/', 'b'])
+
+    def test_element_selector(self):
+        selector = ElementSelector('*')
+        self.assertEqual(list(selector.parts), ['*'])
+
 
 if __name__ == '__main__':
-    import platform
-    header_template = "Test xmlschema XPath with Python {} on {}"
-    header = header_template.format(platform.python_version(), platform.platform())
-    print('{0}\n{1}\n{0}'.format("*" * len(header), header))
-
-    unittest.main()
+    from xmlschema.testing import run_xmlschema_tests
+    run_xmlschema_tests('XPath processor')
