@@ -13,6 +13,7 @@ import os
 import base64
 import json
 import math
+import pathlib
 from decimal import Decimal
 from collections.abc import MutableMapping, MutableSequence, Set
 from textwrap import dedent
@@ -551,7 +552,8 @@ def iter_nested_iterables(obj):
 
 
 class TestDecoding(XsdValidatorTestCase):
-    TEST_CASES_DIR = os.path.join(os.path.dirname(__file__), '../test_cases')
+
+    cases_dir = pathlib.Path(__file__).parent.joinpath('../test_cases')
 
     def check_decode(self, xsd_component, data, expected, **kwargs):
         if isinstance(expected, type) and issubclass(expected, Exception):
@@ -1474,7 +1476,8 @@ class TestDecoding(XsdValidatorTestCase):
         with self.assertRaises(XMLSchemaValidationError) as ctx:
             schema.to_dict(self.casepath('issues/issue_200/issue_200.xml'),
                            path='/na:main/na:item[@doc_id=2]'),
-        self.assertIn('is not an element of the schema', str(ctx.exception))
+        self.assertIn("doesn't select any '{ames}item' element of the schema",
+                      str(ctx.exception))
 
     def test_issue_238__decode_bytes_strings(self):
         with open(self.vh_xml_file, 'rb') as fp:
@@ -1870,9 +1873,5 @@ class TestDecoding11(TestDecoding):
 
 
 if __name__ == '__main__':
-    import platform
-    header_template = "Test xmlschema decoding with Python {} on {}"
-    header = header_template.format(platform.python_version(), platform.platform())
-    print('{0}\n{1}\n{0}'.format("*" * len(header), header))
-
-    unittest.main()
+    from xmlschema.testing import run_xmlschema_tests
+    run_xmlschema_tests('decoding')
