@@ -8,7 +8,7 @@
 #
 # @author Davide Brunato <brunato@sissa.it>
 #
-import unittest
+import pathlib
 
 from xmlschema import XMLSchemaParseError
 from xmlschema.validators import XMLSchema11
@@ -17,6 +17,8 @@ from xmlschema.testing import XsdValidatorTestCase
 
 class TestXsdElements(XsdValidatorTestCase):
 
+    cases_dir = pathlib.Path(__file__).parent.joinpath('../test_cases')
+
     def test_element_ref(self):
         schema = self.check_schema("""
         <xs:element name="node" type="xs:string"/>
@@ -113,9 +115,5 @@ class TestXsd11Elements(TestXsdElements):
 
 
 if __name__ == '__main__':
-    import platform
-    header_template = "Test xmlschema's XSD elements with Python {} on {}"
-    header = header_template.format(platform.python_version(), platform.platform())
-    print('{0}\n{1}\n{0}'.format("*" * len(header), header))
-
-    unittest.main()
+    from xmlschema.testing import run_xmlschema_tests
+    run_xmlschema_tests('XSD elements')
