@@ -19,8 +19,8 @@ try:
 except ImportError:
     lxml_etree = None
 
-from xmlschema import XMLSchema, XMLResource
-from xmlschema.helpers import is_etree_element
+from xmlschema import XMLSchema, XMLResource, iter_errors
+from xmlschema.utils.etree import is_etree_element
 from xmlschema.validators.exceptions import XMLSchemaValidatorError, \
     XMLSchemaNotBuiltError, XMLSchemaParseError, XMLSchemaModelDepthError, \
     XMLSchemaValidationError, XMLSchemaDecodeError, XMLSchemaEncodeError, \
@@ -37,7 +37,7 @@ class TestValidatorExceptions(unittest.TestCase):
 
     def test_exception_init(self):
         with self.assertRaises(ValueError) as ctx:
-            XMLSchemaValidatorError(self.schema, 'unknown error', elem='wrong')
+            XMLSchemaValidatorError(self.schema, 'unknown error', elem='wrong')  # noqa
         self.assertIn("'elem' attribute requires an Element", str(ctx.exception))
 
         error = XMLSchemaNotBuiltError(self.schema, 'schema not built!')
@@ -134,7 +134,7 @@ class TestValidatorExceptions(unittest.TestCase):
         root = lxml_etree.XML('<root a="10"/>')
 
         with self.assertRaises(XMLSchemaValidationError) as ctx:
-            schema.validate(root)
+            schema.validate(root)  # noqa
 
         lines = str(ctx.exception).split('\n')
         self.assertEqual(lines[0], "failed validating {'a': '10'} with XsdAttributeGroup():")
@@ -230,7 +230,7 @@ class TestValidatorExceptions(unittest.TestCase):
 
         root = lxml_etree.XML('<root a="10"/>')
         with self.assertRaises(XMLSchemaValidationError) as ctx:
-            schema.validate(root)
+            schema.validate(root)  # noqa
 
         self.assertEqual(ctx.exception.sourceline, 1)
         self.assertEqual(ctx.exception.root, root)
@@ -245,7 +245,7 @@ class TestValidatorExceptions(unittest.TestCase):
         self.assertIsNone(ctx.exception.root)
         self.assertIsNotNone(ctx.exception.schema_url)
         self.assertEqual(ctx.exception.origin_url, xs.source.url)
-        self.assertIsNone(XMLSchemaValidatorError(None, 'unknown error').origin_url)
+        self.assertIsNone(XMLSchemaValidatorError(None, 'unknown error').origin_url)  # noqa
 
     def test_decode_error(self):
         error = XMLSchemaDecodeError(
@@ -374,7 +374,8 @@ class TestValidatorExceptions(unittest.TestCase):
         self.assertEqual(invalid_child.tag, 'c1')
 
         xml_source = '<a><b1></b1><b2><c1/><c1/></b2></a>'
-        resource = XMLResource(xml_source, lazy=True)
+        resource = XMLResource(xml_source)
+
         errors = list(schema.iter_errors(resource))
         self.assertEqual(len(errors), 3)
         self.assertIsNone(errors[0].invalid_child)
@@ -383,6 +384,15 @@ class TestValidatorExceptions(unittest.TestCase):
         self.assertTrue(is_etree_element(errors[2].invalid_child))
         self.assertEqual(errors[2].invalid_child.tag, 'b2')
 
+        xml_source = '<a><b1></b1><b2><c1/><c1/></b2></a>'
+        resource = XMLResource(xml_source, lazy=True)
+
+        errors = list(schema.iter_errors(resource))
+        self.assertEqual(len(errors), 3)
+        self.assertIsNone(errors[0].invalid_child)
+        self.assertIsNone(errors[1].invalid_child)
+        self.assertIsNone(errors[2].invalid_child)
+
     def test_validation_error_logging(self):
         schema = XMLSchema("""
              <xs:schema xmlns:xs="http://www.w3.org/2001/XMLSchema">
@@ -402,11 +412,32 @@ class TestValidatorExceptions(unittest.TestCase):
             self.assertIn('Collect XMLSchemaDecodeError', ctx.output[0])
             self.assertIn('with traceback:', ctx.output[0])
 
+    @unittest.skipIf(lxml_etree is None, 'lxml is not installed ...')
+    def test_sourceline__issue_456(self):
+        xsd_file = CASES_DIR.joinpath('examples/vehicles/vehicles.xsd')
+        xml_file = CASES_DIR.joinpath('examples/vehicles/vehicles-2_errors.xml')
+        schema = XMLSchema(xsd_file)
+
+        root = lxml_etree.parse(xml_file).getroot()
+
+        errors = [e for e in schema.iter_errors(root)]  # noqa
+        self.assertEqual(len(errors), 2)
+        self.assertEqual(errors[0].sourceline, 4)
+        self.assertEqual(errors[1].sourceline, 9)
+
+        kwargs = dict(schema=schema, lazy=False)
+        errors = [e for e in iter_errors(root, **kwargs)]  # noqa
+        self.assertEqual(len(errors), 2)
+        self.assertEqual(errors[0].sourceline, 4)
+        self.assertEqual(errors[1].sourceline, 9)
+
+        kwargs = dict(iterparse=lxml_etree.iterparse, schema=schema, lazy=True)
+        errors = [e for e in iter_errors(xml_file, **kwargs)]  # noqa
+        self.assertEqual(len(errors), 2)
+        self.assertEqual(errors[0].sourceline, 4)
+        self.assertEqual(errors[1].sourceline, 9)
+
 
 if __name__ == '__main__':
-    import platform
-    header_template = "Test xmlschema's validator exceptions with Python {} on {}"
-    header = header_template.format(platform.python_version(), platform.platform())
-    print('{0}\n{1}\n{0}'.format("*" * len(header), header))
-
-    unittest.main()
+    from xmlschema.testing import run_xmlschema_tests
+    run_xmlschema_tests('validator exceptions')
