@@ -27,6 +27,7 @@ class ModelGroup(XsdGroup):
         self._group: List[Union[ParticleMixin, 'ModelGroup']] = []
         self.content = self._group
         self.model: str = model
+        self.ref = None
 
     def __repr__(self) -> str:
         return '%s(model=%r, occurs=%r)' % (self.__class__.__name__, self.model, self.occurs)
@@ -56,16 +57,16 @@ class TestXsdGroups(unittest.TestCase):
         group = ModelGroup('choice')
 
         group.append(('a',))
-        self.assertListEqual(group[:], [('a',)])
+        self.assertListEqual(group[:], [('a',)])  # noqa
 
         group[0] = ('a', 'b')
-        self.assertListEqual(group[:], [('a', 'b')])
+        self.assertListEqual(group[:], [('a', 'b')])  # noqa
 
         group.append(('c',))
-        self.assertListEqual(group[:], [('a', 'b'), ('c',)])
+        self.assertListEqual(group[:], [('a', 'b'), ('c',)])  # noqa
 
         del group[0]
-        self.assertListEqual(group[:], [('c',)])
+        self.assertListEqual(group[:], [('c',)])  # noqa
 
     def test_is_empty(self):
         group = ModelGroup('all')
@@ -183,7 +184,7 @@ class TestXsdGroups(unittest.TestCase):
         root_group = group = ModelGroup('sequence')
         for k in range(3):
             for _ in range(k + 1):
-                group.append(ParticleMixin())
+                group.append(ParticleMixin())  # noqa
             group.append(ModelGroup('sequence'))
             group = group[-1]
 
@@ -197,7 +198,7 @@ class TestXsdGroups(unittest.TestCase):
         root_group = group = ModelGroup('sequence')
         for k in range(3):
             for _ in range(k + 1):
-                group.append(ParticleMixin())
+                group.append(ParticleMixin())  # noqa
             group.append(ModelGroup('sequence', max_occurs=None))
             group = group[-1]
 
@@ -210,7 +211,7 @@ class TestXsdGroups(unittest.TestCase):
         # Model group with an excessive depth
         root_group = group = ModelGroup('sequence')
         for k in range(16):
-            group.append(ParticleMixin())
+            group.append(ParticleMixin())  # noqa
             group.append(ModelGroup('sequence'))
             group = group[1]
 
@@ -223,7 +224,7 @@ class TestXsdGroups(unittest.TestCase):
         root_group = group = ModelGroup('sequence')
         for k in range(3):
             for _ in range(k + 1):
-                group.append(ParticleMixin())
+                group.append(ParticleMixin())  # noqa
             group.append(ModelGroup('sequence'))
             group = group[-1]
 
@@ -237,7 +238,7 @@ class TestXsdGroups(unittest.TestCase):
         root_group = group = ModelGroup('sequence')
         for k in range(3):
             for _ in range(k + 1):
-                group.append(ParticleMixin())
+                group.append(ParticleMixin())  # noqa
             group.append(ModelGroup('sequence', max_occurs=None))
             group = group[-1]
 
@@ -253,7 +254,7 @@ class TestXsdGroups(unittest.TestCase):
         # Model group with an excessive depth
         root_group = group = ModelGroup('sequence')
         for k in range(16):
-            group.append(ParticleMixin())
+            group.append(ParticleMixin())  # noqa
             group.append(ModelGroup('sequence'))
             group = group[1]
 
@@ -267,7 +268,7 @@ class TestXsdGroups(unittest.TestCase):
         subgroups = []
         for k in range(4):
             for _ in range(k + 1):
-                group.append(ParticleMixin())
+                group.append(ParticleMixin())  # noqa
             group.append(ModelGroup('sequence'))
             subgroups.append(group)
             group = group[-1]
@@ -279,12 +280,12 @@ class TestXsdGroups(unittest.TestCase):
         self.assertListEqual(root_group.get_subgroups(subgroups[-4][0]), subgroups[:-3])
 
         with self.assertRaises(XMLSchemaModelError):
-            root_group.get_subgroups(ParticleMixin())
+            root_group.get_subgroups(ParticleMixin())  # noqa
 
         # Model group with an excessive depth
         root_group = group = ModelGroup('sequence')
         for k in range(18):
-            group.append(ParticleMixin())
+            group.append(ParticleMixin())  # noqa
             group.append(ModelGroup('sequence'))
             group = group[1]
 
@@ -295,7 +296,7 @@ class TestXsdGroups(unittest.TestCase):
         root_group = group = ModelGroup('sequence')
         subgroups = []
         for k in range(4):
-            group.append(ParticleMixin())
+            group.append(ParticleMixin())  # noqa
             group.append(ModelGroup('sequence', max_occurs=10))
             subgroups.append(group)
             group = group[-1]
@@ -313,7 +314,7 @@ class TestXsdGroups(unittest.TestCase):
         root_group = group = ModelGroup('choice')
         subgroups = []
         for k in range(4):
-            group.append(ParticleMixin())
+            group.append(ParticleMixin())  # noqa
             group.append(ModelGroup('choice', max_occurs=10))
             subgroups.append(group)
             group = group[-1]
@@ -324,7 +325,7 @@ class TestXsdGroups(unittest.TestCase):
         root_group = group = ModelGroup('sequence', min_occurs=0)
         subgroups = []
         for k in range(4):
-            group.append(ParticleMixin())
+            group.append(ParticleMixin())  # noqa
             group.append(ModelGroup('sequence', min_occurs=0))
             subgroups.append(group)
             group = group[-1]
@@ -436,10 +437,5 @@ class TestXsdGroups(unittest.TestCase):
 
 
 if __name__ == '__main__':
-    import platform
-
-    header_template = "Test xmlschema's XSD groups with Python {} on {}"
-    header = header_template.format(platform.python_version(), platform.platform())
-    print('{0}\n{1}\n{0}'.format("*" * len(header), header))
-
-    unittest.main()
+    from xmlschema.testing import run_xmlschema_tests
+    run_xmlschema_tests('XSD groups')
