@@ -8,8 +8,7 @@
 #
 # @author Davide Brunato <brunato@sissa.it>
 #
-import unittest
-import os
+import pathlib
 import xml.etree.ElementTree as ElementTree
 
 from xmlschema import XMLSchemaParseError, XMLSchemaValidationError
@@ -19,7 +18,8 @@ from xmlschema.testing import XsdValidatorTestCase
 
 
 class TestXsdIdentities(XsdValidatorTestCase):
-    TEST_CASES_DIR = os.path.join(os.path.dirname(__file__), '../test_cases')
+
+    cases_dir = pathlib.Path(__file__).parent.joinpath('../test_cases')
 
     def test_key_definition(self):
         schema = self.check_schema("""
@@ -462,6 +462,24 @@ class TestXsd11Identities(TestXsdIdentities):
         self.assertIs(schema.elements['secondary_key'].identities[0].ref,
                       schema.elements['primary_key'].identities[-1])
 
+    def test_xsi_type_substitution__issue_446(self):
+        xsd_file = self.casepath('features/substitutions/words.xsd')
+        schema = self.schema_class(xsd_file)
+
+        xml_data = '<words><word>foo</word></words>'
+        self.assertTrue(schema.is_valid(xml_data))
+        xml_data = '<words><word>foo</word><word>foo</word></words>'
+        self.assertFalse(schema.is_valid(xml_data))
+
+        xml_file = self.casepath('features/substitutions/words1.xml')
+        self.assertTrue(schema.is_valid(xml_file))
+        xml_file = self.casepath('features/substitutions/words2.xml')
+        self.assertTrue(schema.is_valid(xml_file))
+        xml_file = self.casepath('features/substitutions/words3.xml')
+        self.assertFalse(schema.is_valid(xml_file))
+        xml_file = self.casepath('features/substitutions/words4.xml')
+        self.assertFalse(schema.is_valid(xml_file))
+
     def test_selector_default_namespace(self):
         schema = self.check_schema("""
             <xs:element name="primary_key" type="xs:string">
@@ -477,9 +495,5 @@ class TestXsd11Identities(TestXsdIdentities):
 
 
 if __name__ == '__main__':
-    import platform
-    header_template = "Test xmlschema's XSD identities with Python {} on {}"
-    header = header_template.format(platform.python_version(), platform.platform())
-    print('{0}\n{1}\n{0}'.format("*" * len(header), header))
-
-    unittest.main()
+    from xmlschema.testing import run_xmlschema_tests
+    run_xmlschema_tests('identities')
