@@ -11,13 +11,13 @@
 """Tests concerning model groups validation"""
 import unittest
 import copy
-import os.path
-import warnings
+import pathlib
 from itertools import zip_longest
+
 from textwrap import dedent
 from typing import Any, Union, List, Optional
 
-from xmlschema import XMLSchema10, XMLSchema11
+from xmlschema import XMLSchema11
 from xmlschema.exceptions import XMLSchemaValueError
 from xmlschema.validators.exceptions import XMLSchemaValidationError, XMLSchemaModelError
 from xmlschema.validators.particles import ParticleMixin
@@ -47,8 +47,7 @@ class ModelGroup(XsdGroup):
 
 class TestModelValidation(XsdValidatorTestCase):
 
-    TEST_CASES_DIR = os.path.join(os.path.dirname(__file__), '../test_cases')
-    schema_class = XMLSchema10
+    cases_dir = pathlib.Path(__file__).parent.joinpath('../test_cases')
 
     # --- Test helper functions ---
 
@@ -1449,7 +1448,8 @@ class TestModelValidation11(TestModelValidation):
 
 
 class TestModelBasedSorting(XsdValidatorTestCase):
-    TEST_CASES_DIR = os.path.join(os.path.dirname(__file__), 'test_cases')
+
+    cases_dir = pathlib.Path(__file__).parent.joinpath('../test_cases')
 
     def test_sort_content(self):
         # test of ModelVisitor's sort_content/iter_unordered_content
@@ -1665,64 +1665,6 @@ class TestModelBasedSorting(XsdValidatorTestCase):
         content = [('X', None), ('B1', 'abc'), ('B2', 10), ('B3', False)]
         self.assertListEqual(list(iter_collapsed_content(content, group)), content)
 
-    def test_deprecated_methods(self):
-        schema = self.get_schema("""
-            <xs:element name="A" type="A_type" />
-            <xs:complexType name="A_type">
-                <xs:sequence maxOccurs="10">
-                    <xs:element name="B1" minOccurs="0" />
-                    <xs:element name="B2" minOccurs="0" />
-                </xs:sequence>
-            </xs:complexType>
-            """)
-
-        group = schema.types['A_type'].content
-        model = ModelVisitor(group)
-        default_namespace = 'http://xmlschema.test/ns'
-        content = [('B1', 1), ('B1', 2), ('B2', 3)]
-
-        with warnings.catch_warnings(record=True) as ctx:
-            warnings.   simplefilter("always")
-            self.assertListEqual(
-                list(model.iter_collapsed_content(content)),
-                [('B1', 1), ('B2', 3), ('B1', 2)]
-            )
-            self.assertEqual(len(ctx), 1, "Expected one deprecation warning")
-            self.assertIn("use iter_collapsed_content()", str(ctx[0].message))
-            self.assertNotIn("Don't provide default_namespace", str(ctx[0].message))
-
-        with warnings.catch_warnings(record=True) as ctx:
-            warnings.simplefilter("always")
-            self.assertListEqual(
-                list(model.iter_collapsed_content(content, default_namespace)),
-                [('B1', 1), ('B2', 3), ('B1', 2)]
-            )
-            self.assertEqual(len(ctx), 1, "Expected one deprecation warning")
-            self.assertIn("use iter_collapsed_content()", str(ctx[0].message))
-            self.assertIn("Don't provide default_namespace", str(ctx[0].message))
-
-        content = [('B2', 1), ('B1', 2), ('B2', 3), ('B1', 4)]
-
-        with warnings.catch_warnings(record=True) as ctx:
-            warnings.   simplefilter("always")
-            self.assertListEqual(
-                list(model.iter_unordered_content(content)),
-                [('B1', 2), ('B2', 1), ('B1', 4), ('B2', 3)]
-            )
-            self.assertEqual(len(ctx), 1, "Expected one deprecation warning")
-            self.assertIn("use iter_unordered_content()", str(ctx[0].message))
-            self.assertNotIn("Don't provide default_namespace", str(ctx[0].message))
-
-        with warnings.catch_warnings(record=True) as ctx:
-            warnings.simplefilter("always")
-            self.assertListEqual(
-                list(model.iter_unordered_content(content, default_namespace)),
-                [('B1', 2), ('B2', 1), ('B1', 4), ('B2', 3)]
-            )
-            self.assertEqual(len(ctx), 1, "Expected one deprecation warning")
-            self.assertIn("use iter_unordered_content()", str(ctx[0].message))
-            self.assertIn("Don't provide default_namespace", str(ctx[0].message))
-
 
 class TestModelPaths(unittest.TestCase):
 
@@ -1791,9 +1733,5 @@ class TestModelPaths(unittest.TestCase):
 
 
 if __name__ == '__main__':
-    import platform
-    header_template = "Test xmlschema's XSD model groups with Python {} on {}"
-    header = header_template.format(platform.python_version(), platform.platform())
-    print('{0}\n{1}\n{0}'.format("*" * len(header), header))
-
-    unittest.main()
+    from xmlschema.testing import run_xmlschema_tests
+    run_xmlschema_tests('XSD model groups')
